import discord
import discord.ext.commands as commands
import discord.app_commands as app_commands
import sys

from astra.connections import AstraDBConnection, AstraHandler
from astra.groups import *

info_text = """
**ASTRA: SE**
A Simple Text Recording Assistant: Special Edition v1.3.1

**New features:**
- Can now target other members to view their jar with optional argument to `jar check`
- Can now view top 20 filthiest mouths in a server with `jar leaderboard`"""

class Astra(commands.Bot):
    def __init__(self, prefix, **kwargs):
        super().__init__(prefix, **kwargs)
        base_cmds = [
            app_commands.Command(name='info',
                                 description='Version, changelog, and other info',
                                 callback=self.info)
        ]
        for cmd in base_cmds:
            self.tree.add_command(cmd)
        
    async def setup_hook(self) -> None:
        await super().setup_hook()
        AstraDBConnection.initialize()
        await self.tree.sync()
        
    async def on_ready(self):
        print('ready', file=sys.stderr)
        
    async def on_message(self, message):
        await AstraHandler.check_profanity(message)
        
    async def info(self, interaction: discord.Interaction):
        await interaction.response.send_message(info_text, ephemeral=True)


import discord
import profanity_check

from astra.connections import AstraDBConnection
from astra.generation import AstraMarkovModel
from astra.base import AbstractPageView

class QuoteView(AbstractPageView):
    
    def __init__(self, *, interaction: discord.Interaction, target: discord.Member, raw: list[tuple[str, int]], perPage: int=5):
        self.target = target
        super().__init__(interaction=interaction, raw=raw, perPage=perPage)
        
    async def build_view(self):
        start, end = (self.page - 1) * self.per_page, self.page * self.per_page
        embed = discord.Embed(title=self.target, description='')
        for (msg, ind) in self._raw[start:end]:
            embed.description += f'#{ind}: "{msg}"\n\n'
        embed.set_author(name=f'Requested by {self.interaction.user}')
        embed.set_footer(text=f'Page {self.page} of {self.total_pages}')
        embed.set_thumbnail(url=self.target.display_avatar.url)
        return embed

class JarView(AbstractPageView):
    
    def __init__(self, *, interaction: discord.Interaction, raw: list[tuple[int, int]], perPage: int=10):
        super().__init__(interaction=interaction, raw=raw, perPage=perPage)
        
    async def build_view(self):
        start, end = (self.page - 1) * self.per_page, self.page * self.per_page
        embed = discord.Embed(title=f'{self.interaction.guild.name} Leaderboard', description='**Showing the top 20**\n')
        for (id, count) in self._raw[start:end]:
            embed.description += f'\n{self.interaction.guild.get_member(id).display_name}: {count} 🪙\n'
        embed.set_author(name=f'Requested by {self.interaction.user}')
        embed.set_footer(text=f'Page {self.page} of {self.total_pages}')
        embed.set_thumbnail(url=(self.interaction.guild.icon.url if self.interaction.guild.icon is not None else None))
        return embed

class AstraHandler:
    
    @staticmethod
    async def add_quote(interaction: discord.Interaction, user: discord.Member | discord.User, msg: str):
        if isinstance(msg, discord.Message):
            msg = msg.clean_content
        if await AstraHandler.does_quote_exist(user, msg):
            await interaction.response.send_message('Quote already present.', ephemeral=True)
        else:
            AstraDBConnection().add_quote(user.id, msg)
            await interaction.response.send_message(f'Quoted {user.mention} saying "{msg}"')
            AstraMarkovModel().initialize_model()
        
    @staticmethod
    async def read_quotes(interaction: discord.Interaction, fromUser: discord.Member | discord.User):
        raw = AstraDBConnection.read_quotes(fromUser.id)
        if len(raw) == 0:
            await interaction.response.send_message('No quotes found.', ephemeral=True)
        else:
            view = QuoteView(interaction=interaction, target=fromUser, raw=raw)
            await view.show()
    
    @staticmethod
    async def does_quote_exist(fromUser: discord.Member | discord.User, withMsg: str | discord.Message, /):
        if isinstance(withMsg, discord.Message):
            withMsg = withMsg.clean_content
        result = AstraDBConnection.search_quote(fromUser.id, withMsg)
        print(result)
        return len(result) > 0
    
    @staticmethod
    async def debug_remove_quote(withIdent: int):
        AstraDBConnection.delete_quote(withIdent)
        
    @staticmethod
    async def check_profanity(forMsg: discord.Message):
        if profanity_check.predict([forMsg.clean_content])[0] == 1:
            AstraDBConnection.incr_jar(forMsg.author.id)
            await forMsg.add_reaction('🪙')
            
    @staticmethod
    async def jar_check(interaction: discord.Interaction, forUser: discord.Member):
        coins = AstraDBConnection.get_jar(forUser.id)
        if forUser.id == interaction.user.id:
            await interaction.response.send_message(f'You have {coins} :coin: in your jar!')
        else:
            await interaction.response.send_message(f'{forUser.mention} has {coins} :coin: in their jar!', allowed_mentions=discord.AllowedMentions.none())
    
    @staticmethod
    async def show_leaderboard(interaction: discord.Interaction):
        members = [m.id for m in interaction.guild.members]
        list = AstraDBConnection.query_leaderboard(members)
        if len(list) == 0:
            await interaction.response.send_message('No swears in this server, yet.', ephemeral=True)
        else:
            await JarView(interaction=interaction, raw=list).show()


import discord
import discord.ext.commands as commands

from astra.connections import AstraHandler
from discord import app_commands
from typing import Optional

class JarGroup(commands.GroupCog, name='jar', group_name='jar'):
    def __init__(self, bot: commands.Bot):
        self.bot = bot
        
    @app_commands.command(name='check', description="Check a user's swear jar")
    @app_commands.describe(target='The user to query. If empty, defaults to sender.')   
    async def jar(self, interaction: discord.Interaction, target: Optional[discord.Member]=None):
        if target is None:
            target = interaction.user
        await AstraHandler.jar_check(interaction, target)
    
    @app_commands.command(name='leaderboard', description='Get the server swear jar leaderboard.')
    async def lb(self, interaction: discord.Interaction):
        await AstraHandler.show_leaderboard(interaction)


import discord

from abc import ABCMeta, abstractmethod
from typing import Any

class AbstractPageView(discord.ui.View, metaclass=ABCMeta):
    def __init__(self, *, interaction: discord.Interaction, raw: list[Any], perPage: int=5):
        self.interaction = interaction
        self._raw = raw
        self.total_pages = (len(raw) // perPage) + 1 if (len(raw) % perPage != 0) else (len(raw) // perPage)
        self.page = 1
        self.per_page = perPage
        super().__init__(timeout=60)
    
    async def show(self):
        embed = await self.build_view()
        await self.update_buttons()
        await self.interaction.response.send_message(embed=embed, view=self)
        
    @abstractmethod
    async def build_view(self):
        pass
    
    async def update_view(self, interaction: discord.Interaction):
        embed = await self.build_view()
        await self.update_buttons()
        await interaction.response.edit_message(embed=embed, view=self)
        
    async def update_buttons(self):
        self.children[0].disabled = (self.total_pages == 1 or self.page <= 2)
        self.children[1].disabled = (self.total_pages == 1 or self.page == 1)
        self.children[2].disabled = (self.total_pages == 1 or self.page == self.total_pages)
        self.children[3].disabled = (self.total_pages == 1 or self.page >= self.total_pages - 1)
        
    async def interaction_check(self, interaction: discord.Interaction) -> bool:
        return interaction.user.id == self.interaction.user.id
       
    @discord.ui.button(style=discord.ButtonStyle.secondary, emoji='⏮️')
    async def first(self, interaction: discord.Interaction, button: discord.Button):
        self.page = 1
        await self.update_view(interaction)
    
    @discord.ui.button(style=discord.ButtonStyle.primary, emoji='⏪')
    async def prev(self, interaction: discord.Interaction, button: discord.Button):
        self.page -= 1
        await self.update_view(interaction)
        
    @discord.ui.button(style=discord.ButtonStyle.primary, emoji='⏩')
    async def succ(self, interaction: discord.Interaction, button: discord.Button):
        self.page += 1
        await self.update_view(interaction)
        
    @discord.ui.button(style=discord.ButtonStyle.secondary, emoji='⏭️')
    async def last(self, interaction: discord.Interaction, button: discord.Button):
        self.page = self.total_pages
        await self.update_view(interaction)
        
    async def on_timeout(self):
        message = await self.interaction.original_response()
        await message.edit(view=None)


import discord
import discord.ext.commands as commands

from discord import app_commands

from astra.connections import AstraHandler
from astra.generation import AstraMarkovModel

@app_commands.allowed_contexts(guilds=True, dms=False, private_channels=True)
class QuoteGroup(commands.GroupCog, name='quote', group_name='quote'):
    def __init__(self, bot: commands.Bot):
        self.bot = bot
        menus = [
            app_commands.ContextMenu(
                name='Quote Message',
                callback=self.quote_msg),
            app_commands.ContextMenu(
                name='List Quotes',
                callback=self.quote_list)]
        for menu in menus:
            self.bot.tree.add_command(menu)
    
    async def quote_msg(self, interaction: discord.Interaction, msg: discord.Message):
        await AstraHandler.add_quote(interaction, msg.author, msg)
        
    @app_commands.command(name='add', description='Add a quote')
    @app_commands.describe(user='The user to quote', msg='The quote')
    async def quote_msg_cmd(self, interaction: discord.Interaction, user: discord.User | discord.Member, msg: str):
        await AstraHandler.add_quote(interaction, user, msg)
        
    async def quote_list(self, interaction: discord.Interaction, user: discord.Member):
        await AstraHandler.read_quotes(interaction, user)
        
    @app_commands.command(name='list', description='List quotes')
    @app_commands.describe(user='Target user')
    async def quote_list_cmd(self, interaction: discord.Interaction, user: discord.User | discord.Member):
        await AstraHandler.read_quotes(interaction, user)
        
    @app_commands.command(name='gen', description='Generate a quote')
    async def quote_gen(self, interaction: discord.Interaction):
        sentence = AstraMarkovModel().make_sentence()
        await interaction.response.send_message(sentence)


import discord
import discord.ext.commands as commands
from discord import app_commands

from astra.connections import AstraHandler

@app_commands.allowed_contexts(guilds=True, dms=True, private_channels=True)
class DebugGroup(commands.GroupCog, name='debug', group_name='debug'):
    def __init__(self, bot: commands.Bot):
        self.bot = bot
        
    @app_commands.command(name='remove', description='Remove a quote')
    @app_commands.describe(ident='ID of quote')
    @app_commands.default_permissions(administrator=True)
    @app_commands.checks.has_permissions(administrator=True)
    async def remove_quote(self, interaction: discord.Interaction, ident: int):
        await AstraHandler.debug_remove_quote(ident)
        await interaction.response.send_message('Done!', ephemeral=True)